// PWM_setup.s - Implementa la rutina PWM_setup

// Directivas para el ensamblador
// ------------------------------
.syntax unified
.cpu cortex-m3

// Declaración de funciones externas y exportadas
// ----------------------------------------------
.global PWM_setup               @ Para que la función 'PWM_setup' sea
.type PWM_setup, %function      @   accesible desde otros módulos

// Declaración de constantes
// -------------------------
// Direcciones base, offsets y máscaras de los registros de los PIO
.equ PIOC,      0x400E1200  @ Dirección base del puerto PIOC
.equ PIO_PDR,   0x004       @ Offset Pio Disable Register
.equ PIO_PSR,   0x008       @ Offset Pio Status Register
.equ PIO_ABSR,  0x070       @ Offset A B Select Register
// Direcciones base, offsets y máscaras de los registros del reloj del sistema
.equ PMC_BASE,  0x400E0600  @ Dirección base del PMC
.equ PMC_WPMR,  0x0E4       @ Offset del PMC Write Protect Mode Register
.equ PMC_PCER1, 0x100       @ Offset del Peripheral Clock Enable Register
.equ PMC_PCSR1, 0x108       @ Offset del Peripheral Clock Status Register
.equ PWM_MASK,  0x00000010  @ Máscara para activar el reloj del PWM
.equ FRE_MASK,  0x504D4300  @ Máscara para desproteger la escritura del PMC_PCER1
.equ BLQ_MASK,  0x504D4301  @ Máscara para bloquear la escritura del PMC_PCER1
// Dirección base y offsets de los registros del PWM
.equ PWM_BASE,  0x40094000  @ Dirección base de E/S del PWM
.equ PWM_CLK,   0x000       @ Offset PWM Clock Register
.equ PWM_ENA,   0x004       @ Offset PWM Enable Register
.equ PWM_SR,    0x00C       @ Offset PWM Status Register
.equ PWM_WPCR,  0x0E4       @ Offset PWM Write Protect Control Register
.equ PWM_WPSR,  0x0E8       @ Offset PWM Write Protect Status Register
.equ PWM_CMR5,  0x2A0       @ Offset PWM Channel Mode Register (channel 5)
.equ PWM_CDTY5, 0x2A4       @ Offset PWM Channel Duty Cycle Register (channel 5)
.equ PWM_CPRD5, 0x2AC       @ Offset PWM Channel Period Register (channel 5)
// --
.equ ID_PIOB,   0x00C       @ ID del PIOB para el reloj
.equ RMSK,      0x00400000  @ LED rojo en el pin 8 -> bit 22 de PIOC. Señal PWM PWML5
.equ PULMSK,    0x08000000  @ Pulsador en el pin 13 -> bit 27 de PIOB
// Valores de configuración a escribir en los registros del PWM
.equ ENA_MSK,   0x00000020  @ Máscara para activar el canal PWM seleccionado (el 5)
.equ CMR,       0x0000000A  @ Para seleccionar MCK/1024 como reloj base para generar la señal PWM
.equ CPRD,      0x000008FC  @ Para determinar el periodo de la señal PWM
.equ CDTY,      0x00000000  @ Para determinar el ciclo de trabajo


// Comienzo de la zona de datos
// ----------------------------
.data
pio_absr_str: .asciz "  PIO_ABSR (PIO AB Select Register) ................ "
pio_psr_str:  .asciz "  PIO_PSR (PIO Status Register) .................... "
pmc_pcer_str: .asciz "  PMC_PCER (PMC Peripheral Clock Enable Register) .. "
pwm_clk_str:  .asciz "  PWM_CLK (PWM Clock Register) ..................... "
pwm_cmr_str:  .asciz "  PWM_CMR (PWM Channel Mode Register)............... "
pwm_sr_str:   .asciz "  PWM_SR (PWM Status Register) ..................... "
pwm_cprd_str: .asciz "  PWM_CPRD (PWM Channel Period Register) ........... "
pwm_cdty_str: .asciz "  PWM_CDTY (PWM Channel Duty Cycle Register) ....... "


// Comienzo de la zona de código
// -----------------------------
.text

/*
   Subrutina 'PWM_setup'
     Configura el pin correspondiente al LED Rojo para que muestre la salida del PWM.
     Configura el controlador de PWM para generar de 
   Parámetros de entrada:
     No tiene.
   Parámetro de salida:
     r0: el nivel de intensidad luminosa del LED rojo.
*/
.thumb_func
PWM_setup:
  push  {r4-r7, lr}          @ Apila de r4 a r7 y LR

  // Asignación del pin del LED al PWM
  ldr   r4, =PIOC            @ r4 <- dirección base del controlador PIOC
  ldr   r5, =RMSK            @ r5 <- bit 22 activo (LED rojo del RGB)
  str   r5, [r4, #PIO_ABSR]  @ Selecciona el periférico B (PWM) para el bit 22
  str   r5, [r4, #PIO_PDR]   @ Asocia el bit 22 al periférico seleccionado (no a la GPIO)

  // Muestra el valor de PIO_ABSR
  ldr   r0, =pio_absr_str
  bl    printString
  mov   r0, r5
  bl    printWord
  bl    printNL

  // Muestra el valor de PIO_PSR
  ldr   r0, =pio_psr_str
  bl    printString
  ldr   r0, [r4, #PIO_PSR]   @ r0 <- Contenido del PIO_PSR del PIOC
  bl    printWord
  bl    printNL

  // Activación de la señal de reloj del generador PWM en el PMC
  ldr   r4, =PMC_BASE        @ r4 <- dirección base del controlador PMC
  ldr   r5, =FRE_MASK        @ r5 <- Máscara de desprotección de escritura de los registros del PMC
  str   r5, [r4, #PMC_WPMR]  @ Desprotege el acceso a los registros de control del PMC
  // --
  ldr   r6, [r4, #PMC_PCSR1] @ r6 <- contenido del registro de estado
  ldr   r0, =PWM_MASK        @ r0 <- Máscara de activación del reloj del PWM
  orr   r6, r0               @ r6 <- SR con bit del reloj del PWM activado
  str   r6, [r4, #PMC_PCER1] @ Activa el reloj del PWM
  // --
  ldr   r5, =BLQ_MASK        @ r5 <- Máscara de protección contra escritura de los registros del PMC
  str   r5, [r4, #PMC_WPMR]  @ Protege el acceso a los registros de control del PMC

  // Muestra el valor de PMC_PCER1 (en r6)
  ldr   r0, =pmc_pcer_str
  bl    printString
  mov   r0, r6
  bl    printWord
  bl    printNL
   
  // Configuración de los parámetros del PWM
  ldr   r4, =PWM_BASE        @ r4 <- dirección base del controlador PWM
  // PWM_CMR (Determina el reloj utilizado como base de la señal PWM)
  ldr   r5, =CMR
  str   r5, [r4, #PWM_CMR5]
  ldr   r0, =pwm_cmr_str
  bl    printString
  mov   r0, r5
  bl    printWord
  bl    printNL
  // PWM_CPRD5 (Multiplicador del periodo de la señal PWM)
  ldr   r5, =CPRD
  str   r5, [r4, #PWM_CPRD5]
  ldr   r0, =pwm_cprd_str
  bl    printString
  mov   r0, r5
  bl    printWord
  bl    printNL
  // PWM_CDTY5 (Ciclo de trabajo)
  ldr   r5, =CDTY
  str   r5, [r4, #PWM_CDTY5]
  ldr   r0, =pwm_cdty_str
  bl    printString
  mov   r0, r5
  bl    printWord
  bl    printNL
  // Muestra PWM_SR
  ldr   r0, =pwm_sr_str
  bl    printString
  ldr   r0, [r4, #PWM_SR]
  bl    printWord
  bl    printNL
  // PWM_ENA (Habilita PWM en los pines indicados)
  ldr   r5, =ENA_MSK
  str   r5, [r4, #PWM_ENA]

  // Retorna
  mov   r0, r5
  pop   {r4-r7, pc}          @ Desapila r4 a r7 y vuelve
.end

